@@ -12,7 +12,7 @@ import cookielib
 from bs4 import BeautifulSoup
 import tempfile
 from os import path
-import version
+import _version
 
 class CourseraDownloader(object):
     """
@@ -20,8 +20,13 @@ class CourseraDownloader(object):
     use offline.
 
     https://github.com/dgorissen/coursera-dl
-    """
 
+    :param username: username
+    :param password: password
+    :keyword proxy: http proxy, eg: foo.bar.com:1234
+    :keyword parser: xml parser (defaults to lxml)
+    :keyword ignorefiles: comma separated list of file extensions to skip (e.g., "ppt,srt")
+    """
     BASE_URL =    'https://class.coursera.org/%s'
     HOME_URL =    BASE_URL + '/class/index'
     LECTURE_URL = BASE_URL + '/lecture/index'
@@ -29,18 +34,19 @@ class CourseraDownloader(object):
     AUTH_URL =    BASE_URL + "/auth/auth_redirector?type=login&subtype=normal"
     LOGIN_URL =   "https://www.coursera.org/maestro/api/user/login"
 
+    #see http://www.crummy.com/software/BeautifulSoup/bs4/doc/#installing-a-parser
     DEFAULT_PARSER = "lxml"
 
-    def __init__(self,username,password,proxy=None,parser=DEFAULT_PARSER):
-        """
-        Requires your coursera username and password. 
-        You can also specify the parser to use (defaults to lxml)
-        see http://www.crummy.com/software/BeautifulSoup/bs4/doc/#installing-a-parser
-        """
+    def __init__(self,username,password,proxy=None,parser=DEFAULT_PARSER,ignorefiles=None):
         self.username = username
         self.password = password
         self.parser = parser
 
+        # Split "ignorefiles" argument on commas, strip, remove prefixing dot
+        # if there is one, and filter out empty tokens.
+        self.ignorefiles =  [x.strip()[1:] if x[0]=='.' else x.strip()
+                             for x in ignorefiles.split(',') if len(x)]
+
         self.browser = None
         self.proxy = proxy
 
@@ -48,6 +54,7 @@ class CourseraDownloader(object):
         """
         Automatically generate a cookie file for the coursera site.
         """
+        #TODO: use proxy here
         hn,fn = tempfile.mkstemp()
         cookies = cookielib.LWPCookieJar()
         handlers = [
@@ -156,16 +163,39 @@ class CourseraDownloader(object):
             # for each lecture in a weekly class
             classNames = []
             for li in lis:
-                className = sanitise_filename(li.a.text)
-                classNames.append(className)
+                # the name of this lecture/class
+                className = li.a.text.strip()
+
+                # Many class names have the following format: 
+                #   "Something really cool (12:34)"
+                # If the class name has this format, replace the colon in the
+                # time with a hyphen.
+                if re.match(".+\(\d?\d:\d\d\)$",className):
+                    head,sep,tail = className.rpartition(":")
+                    className = head  + "-" + tail
+
+                classNames.append(sanitise_filename(className))
                 classResources = li.find('div', {'class':'course-lecture-item-resource'})
 
                 hrefs = classResources.findAll('a')
 
-                # for each resource of that lecture (slides, pdf, ...)
-                # (dont set a filename here, that will be inferred from the week
-                # titles)
-                resourceLinks = [ (h['href'],None) for h in hrefs]
+                # collect the resources for a particular lecture (slides, pdf,
+                # links,...)
+                resourceLinks = []
+
+                for a in hrefs:
+                    # get the hyperlink itself
+                    h = a['href']
+
+                    # Sometimes the raw, uncompresed source videos are available as
+                    # well. Don't download them as they are huge and available in
+                    # compressed form anyway.
+                    if h.find('source_videos') > 0:
+                        print "   - will skip raw source video " + h
+                    else:
+                        # Dont set a filename here, that will be inferred from the week
+                        # titles
+                        resourceLinks.append( (h,None) )
  
                 # check if the video is included in the resources, if not, try
                 # do download it directly
@@ -173,7 +203,7 @@ class CourseraDownloader(object):
                 if not hasvid:
                     ll = li.find('a',{'class':'lecture-link'})
                     lurl = ll['data-modal-iframe']
-                    bb = self.load_page(lurl)
+                    bb = self.browser.open(lurl)
                     bb = BeautifulSoup(p,self.parser)
                     vobj = bb.find('source',type="video/mp4")
 
@@ -215,12 +245,20 @@ class CourseraDownloader(object):
         # build the absolute path we are going to write to
         fname = target_fname or filename_from_header(headers) or filename_from_url(url)
 
-        filepath = os.path.join(target_dir,fname)
+        # split off the extension
+        _,ext = path.splitext(fname)
+
+        # check if we should skip it (remember to remove the leading .)
+        if ext and ext[1:] in self.ignorefiles:
+            print '    - skipping "%s" (extension ignored)' % fname 
+            return
+
+        filepath = path.join(target_dir,fname)
 
         dl = True
-        if os.path.exists(filepath):
+        if path.exists(filepath):
             if clen > 0: 
-                fs = os.path.getsize(filepath)
+                fs = path.getsize(filepath)
                 delta = clen - fs
 
                 # all we know is that the current filesize may be shorter than it should be and the content length may be incorrect
@@ -266,16 +304,16 @@ class CourseraDownloader(object):
             weeklyTopics.reverse()
             print "* Sections reversed"
 
-        course_dir = os.path.abspath(os.path.join(dest_dir,cname))
+        course_dir = path.abspath(path.join(dest_dir,cname))
 
         # ensure the target dir exists
-        if not os.path.exists(course_dir):
+        if not path.exists(course_dir):
             os.mkdir(course_dir)
 
         print "* " + cname + " will be downloaded to " + course_dir
 
         # ensure the course directory exists
-        if not os.path.exists(course_dir):
+        if not path.exists(course_dir):
             os.makedirs(course_dir)
 
         # download the standard pages
@@ -292,8 +330,8 @@ class CourseraDownloader(object):
             # ensure the week dir exists
             # add a numeric prefix to the week directory name to ensure chronological ordering
             wkdirname = str(j).zfill(2) + " - " + weeklyTopic
-            wkdir = os.path.join(course_dir,wkdirname)
-            if not os.path.exists(wkdir):
+            wkdir = path.join(course_dir,wkdirname)
+            if not path.exists(wkdir):
                 os.makedirs(wkdir)
 
             weekClasses = allClasses[weeklyTopic]
@@ -309,8 +347,8 @@ class CourseraDownloader(object):
 
                 # ensure the class dir exists
                 clsdirname = str(i).zfill(2) + " - " + className
-                clsdir = os.path.join(wkdir,clsdirname)
-                if not os.path.exists(clsdir): 
+                clsdir = path.join(wkdir,clsdirname)
+                if not path.exists(clsdir): 
                     os.makedirs(clsdir)
 
                 print "  - Downloading resources for " + className
@@ -363,7 +401,7 @@ def filename_from_url(url):
         fname = fname.split('/')[-1]
 
     # add an extension if none
-    ext = os.path.splitext(fname)[1]
+    ext = path.splitext(fname)[1]
     if len(ext) < 1 or len(ext) > 5: fname += ".html"
 
     # remove any illegal chars and return
@@ -391,7 +429,7 @@ def sanitise_filename(fileName):
     max = 250
 
     # split off extension, trim, and re-add the extension
-    fn,ext = os.path.splitext(s)
+    fn,ext = path.splitext(s)
     s = fn[:max-len(ext)] + ext
 
     return s
@@ -466,13 +504,14 @@ def main():
     parser.add_argument("-u", dest='username', type=str, required=True, help='coursera.org username')
     parser.add_argument("-p", dest='password', type=str, help='coursera.org password')
     parser.add_argument("-d", dest='dest_dir', type=str, default=".", help='destination directory where everything will be saved')
+    parser.add_argument("-n", dest='ignorefiles', type=str, default="", help='comma-separated list of file extensions to skip, e.g., "ppt,srt,pdf"')
     parser.add_argument("-q", dest='parser', type=str, default=CourseraDownloader.DEFAULT_PARSER,
                         help="the html parser to use, see http://www.crummy.com/software/BeautifulSoup/bs4/doc/#installing-a-parser")
     parser.add_argument("-x", dest='proxy', type=str, default=None, help="proxy to use, e.g., foo.bar.com:3125")
     parser.add_argument("--reverse-sections", dest='reverse', action="store_true",
                         default=False, help="download and save the sections in reverse order")
     parser.add_argument('course_names', nargs="+", metavar='<course name>',
-                        type=str, help='one or more course names (from the url)')
+                        type=str, help='one or more course names from the url (e.g., comnets-2012-001)')
     args = parser.parse_args()
 
     # check the parser
@@ -483,14 +522,14 @@ def main():
     else:
         pass
 
-    print "Coursera-dl v%s (%s)" % (version.VERSION,parser)
+    print "Coursera-dl v%s (%s)" % (_version.__version__,parser)
 
     # prompt the user for his password if not specified
     if not args.password:
         args.password = getpass.getpass()
 
     # instantiate the downloader class
-    d = CourseraDownloader(args.username,args.password,proxy=args.proxy,parser=parser)
+    d = CourseraDownloader(args.username,args.password,proxy=args.proxy,parser=parser,ignorefiles=args.ignorefiles)
     
     # authenticate, only need to do this once but need a classaname to get hold
     # of the csrf token, so simply pass the first one
